#include <dbus/dbus-glib.h>

#include "self.h"
#include "common.h"
#include "signals-marshal.h"

/* Beginning of a classic GObject declaration */

typedef struct Self Self;
typedef struct SelfClass SelfClass;

GType self_get_type (void);

struct Self
{
  GtkFrame parent;

  DBusGProxy *dbus_object;
  GtkWidget *name;
  GtkWidget *location;
  GtkWidget *comment;
  GtkWidget *url;
};

struct SelfClass
{
  GtkFrameClass parent;
};

#define SELF_TYPE (self_get_type ())
#define SELF(object) (G_TYPE_CHECK_INSTANCE_CAST ((object), SELF_TYPE, Self))
#define SELF_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), SELF_TYPE, SelfClass))
#define IS_SELF(object) (G_TYPE_CHECK_INSTANCE_TYPE ((object), SELF_TYPE))
#define IS_SELF_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), SELF_TYPE))

G_DEFINE_TYPE(Self, self, GTK_TYPE_FRAME);

/* End of a classic GObject declaration */

/* Declaration of the methods of this object */

/* Declaration of some helper functions */
static void update_button_callback (GtkWidget *widget, gpointer data);
static void shutdown_button_callback (GtkWidget *widget, gpointer data);

/* Implementation */
static void
self_init (Self *self)
{
  DBusGConnection *bus = NULL;
  GError *error = NULL;
  GtkWidget *vbox = NULL;
  GtkWidget *table = NULL;
  GtkWidget *label = NULL;
  GtkWidget *hbox = NULL;
  GtkWidget *button = NULL;

  bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  if (bus == NULL) {

    print_error ("SELF", "Couldn't connect to session bus", error);
    return;
  }

  self->dbus_object = dbus_g_proxy_new_for_name (bus,
						 "net.gnomemeeting.instance",
						 "/net/gnomemeeting/instance",
						 "net.gnomemeeting.self");

  if (self->dbus_object == NULL) {

    print_error ("SELF", "Failed to get name owner", error);
    return;
  }

  table = gtk_table_new (4, 2, FALSE);
  
  label = gtk_label_new ("Name: ");
  gtk_table_attach (GTK_TABLE (table), label,
		    0, 1, 0, 1,
		    0, 0, 0, 0);
  
  label = gtk_label_new ("Location: ");
  gtk_table_attach (GTK_TABLE (table), label,
		    0, 1, 1, 2,
		    0, 0, 0, 0);
  
  label = gtk_label_new ("Comment: ");
  gtk_table_attach (GTK_TABLE (table), label,
		    0, 1, 2, 3,
		    0, 0, 0, 0);
  
  label = gtk_label_new ("SIP local address: ");
  gtk_table_attach (GTK_TABLE (table), label,
		    0, 1, 3, 4,
		    0, 0, 0, 0);
  
  label = gtk_label_new (" ---- ");
  gtk_table_attach (GTK_TABLE (table), label,
		    1, 2, 0, 1,
		    0, 0, 0, 0);
  self->name = label;
  
  label = gtk_label_new (" ---- ");
  gtk_table_attach (GTK_TABLE (table), label,
		    1, 2, 1, 2,
		    0, 0, 0, 0);
  self->comment = label;
  
  label = gtk_label_new (" ---- ");
  gtk_table_attach (GTK_TABLE (table), label,
		    1, 2, 2, 3,
		    0, 0, 0, 0);
  self->location = label;
  
  label = gtk_label_new (" ---- ");
  gtk_table_attach (GTK_TABLE (table), label,
		    1, 2, 3, 4,
		    0, 0, 0, 0);
  self->url = label;

  hbox = gtk_hbox_new (FALSE, 5);

  button = gtk_button_new_with_label ("Shutdown");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (shutdown_button_callback), self);
  gtk_container_add (GTK_CONTAINER (hbox), button);

  button = gtk_button_new_with_label ("Update");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (update_button_callback), self);
  gtk_container_add (GTK_CONTAINER (hbox), button);

  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (vbox), table);
  gtk_container_add (GTK_CONTAINER (vbox), hbox);

  gtk_container_add (GTK_CONTAINER (self), vbox);

}

static void
self_class_init (SelfClass *klass)
{
  /* nothing to do */
}

static void
update_button_callback (GtkWidget *widget, gpointer data)
{
  Self *self = SELF (data);
  GError *error = NULL;
  char *str = NULL;

  if (!dbus_g_proxy_call (self->dbus_object, "GetName", &error,
                          G_TYPE_INVALID,
                          G_TYPE_STRING, &str,
                          G_TYPE_INVALID)) {

    print_error ("SELF", "Couldn't GetName", error);
    return;
  }
  gtk_label_set_text (GTK_LABEL (self->name), str);
  g_free (str);

  if (!dbus_g_proxy_call (self->dbus_object, "GetComment", &error,
                          G_TYPE_INVALID,
                          G_TYPE_STRING, &str,
                          G_TYPE_INVALID)) {

    print_error ("SELF", "Couldn't GetComment", error);
    return;
  }
  gtk_label_set_text (GTK_LABEL (self->comment), str);
  g_free (str);

  if (!dbus_g_proxy_call (self->dbus_object, "GetLocation", &error,
                          G_TYPE_INVALID,
                          G_TYPE_STRING, &str,
                          G_TYPE_INVALID)) {

    print_error ("SELF", "Couldn't GetLocation", error);
    return;
  }
  gtk_label_set_text (GTK_LABEL (self->location), str);
  g_free (str);

  if (!dbus_g_proxy_call (self->dbus_object, "GetLocalAddress", &error,
			  G_TYPE_STRING, "SIP",
                          G_TYPE_INVALID,
                          G_TYPE_STRING, &str,
                          G_TYPE_INVALID)) {

    print_error ("SELF", "Couldn't GetLocalAddress", error);
    return;
  }
  gtk_label_set_text (GTK_LABEL (self->url), str);
  g_free (str);
}

static void
shutdown_button_callback (GtkWidget *widget, gpointer data)
{
  Self *self = SELF (data);

  dbus_g_proxy_call_no_reply (self->dbus_object, "Shutdown", G_TYPE_INVALID);
}

/* implementation of the public functions */
GtkWidget *
self_new (void)
{
  Self *result = NULL;

  result = SELF (gtk_object_new (SELF_TYPE,
				     "label", "Self",
				     NULL));

  return GTK_WIDGET(result);
}
